@@ -25,14 +25,7 @@
  *    Xiang Haihao <haihao.xiang@intel.com>
  *
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
-
-#ifndef HAVE_GEN_AVC_SURFACE
-#define HAVE_GEN_AVC_SURFACE 1
-#endif
+#include "sysdeps.h"
 
 #include "intel_batchbuffer.h"
 #include "intel_driver.h"
@@ -43,6 +36,7 @@
 #include "i965_media_h264.h"
 #include "i965_media.h"
 #include "i965_decoder_utils.h"
+#include "intel_media.h"
 
 static void
 i965_avc_bsd_init_avc_bsd_surface(VADriverContextP ctx, 
@@ -121,8 +115,6 @@ i965_avc_bsd_img_state(VADriverContextP ctx,
     assert(decode_state->pic_param && decode_state->pic_param->buffer);
     pic_param = (VAPictureParameterBufferH264 *)decode_state->pic_param->buffer;
 
-    assert(!(pic_param->CurrPic.flags & VA_PICTURE_H264_INVALID));
-
     if (pic_param->CurrPic.flags & VA_PICTURE_H264_TOP_FIELD)
         img_struct = 1;
     else if (pic_param->CurrPic.flags & VA_PICTURE_H264_BOTTOM_FIELD)
@@ -379,11 +371,11 @@ i965_avc_bsd_slice_state(VADriverContextP ctx,
 
 static void
 i965_avc_bsd_buf_base_state(VADriverContextP ctx,
+                            struct decode_state *decode_state,
                             VAPictureParameterBufferH264 *pic_param, 
                             VASliceParameterBufferH264 *slice_param,
                             struct i965_h264_context *i965_h264_context)
 {
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct intel_batchbuffer *batch = i965_h264_context->batch;
     struct i965_avc_bsd_context *i965_avc_bsd_context;
     int i, j;
@@ -416,7 +408,9 @@ i965_avc_bsd_buf_base_state(VADriverContextP ctx,
         OUT_BCS_BATCH(batch, 0);
 
     for (i = 0; i < ARRAY_ELEMS(i965_h264_context->fsid_list); i++) {
-        if (i965_h264_context->fsid_list[i].surface_id != VA_INVALID_ID) {
+        if (i965_h264_context->fsid_list[i].surface_id != VA_INVALID_ID &&
+            i965_h264_context->fsid_list[i].obj_surface &&
+            i965_h264_context->fsid_list[i].obj_surface->private_data) {
             int found = 0;
             for (j = 0; j < ARRAY_ELEMS(pic_param->ReferenceFrames); j++) {
                 va_pic = &pic_param->ReferenceFrames[j];
@@ -431,30 +425,21 @@ i965_avc_bsd_buf_base_state(VADriverContextP ctx,
             }
 
             assert(found == 1);
-
-            if (!(va_pic->flags & VA_PICTURE_H264_INVALID)) {
-                obj_surface = SURFACE(va_pic->picture_id);
-                assert(obj_surface);
-                avc_bsd_surface = obj_surface->private_data;
+            obj_surface = i965_h264_context->fsid_list[i].obj_surface;
+            avc_bsd_surface = obj_surface->private_data;
             
-                if (avc_bsd_surface == NULL) {
-                    OUT_BCS_BATCH(batch, 0);
-                    OUT_BCS_BATCH(batch, 0);
-                } else {
-                    OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_top,
-                                  I915_GEM_DOMAIN_INSTRUCTION, 0,
-                                  0);
+            OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_top,
+                          I915_GEM_DOMAIN_INSTRUCTION, 0,
+                          0);
 
-                    if (avc_bsd_surface->dmv_bottom_flag == 1)
-                        OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_bottom,
-                                      I915_GEM_DOMAIN_INSTRUCTION, 0,
-                                      0);
-                    else
-                        OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_top,
-                                      I915_GEM_DOMAIN_INSTRUCTION, 0,
-                                      0);
-                }
-            } 
+            if (avc_bsd_surface->dmv_bottom_flag == 1)
+                OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_bottom,
+                              I915_GEM_DOMAIN_INSTRUCTION, 0,
+                              0);
+            else
+                OUT_BCS_RELOC(batch, avc_bsd_surface->dmv_top,
+                              I915_GEM_DOMAIN_INSTRUCTION, 0,
+                              0);
         } else {
             OUT_BCS_BATCH(batch, 0);
             OUT_BCS_BATCH(batch, 0);
@@ -462,9 +447,7 @@ i965_avc_bsd_buf_base_state(VADriverContextP ctx,
     }
 
     va_pic = &pic_param->CurrPic;
-    assert(!(va_pic->flags & VA_PICTURE_H264_INVALID));
-    obj_surface = SURFACE(va_pic->picture_id);
-    assert(obj_surface);
+    obj_surface = decode_state->render_object;
     obj_surface->flags &= ~SURFACE_REF_DIS_MASK;
     obj_surface->flags |= (pic_param->pic_fields.bits.reference_pic_flag ? SURFACE_REFERENCED : 0);
     i965_check_alloc_surface_bo(ctx, obj_surface, 0, VA_FOURCC('N','V','1','2'), SUBSAMPLE_YUV420);
@@ -825,120 +808,6 @@ i965_avc_bsd_phantom_slice(VADriverContextP ctx,
     i965_avc_bsd_object(ctx, decode_state, pic_param, NULL, 0, i965_h264_context);
 }
 
-static void
-i965_avc_bsd_frame_store_index(VADriverContextP ctx,
-                               VAPictureParameterBufferH264 *pic_param,
-                               struct i965_h264_context *i965_h264_context)
-{
-    struct i965_driver_data *i965 = i965_driver_data(ctx);
-    int i, j;
-
-    assert(ARRAY_ELEMS(i965_h264_context->fsid_list) == ARRAY_ELEMS(pic_param->ReferenceFrames));
-
-    for (i = 0; i < ARRAY_ELEMS(i965_h264_context->fsid_list); i++) {
-        int found = 0;
-
-        if (i965_h264_context->fsid_list[i].surface_id == VA_INVALID_ID)
-            continue;
-
-        for (j = 0; j < ARRAY_ELEMS(pic_param->ReferenceFrames); j++) {
-            VAPictureH264 *ref_pic = &pic_param->ReferenceFrames[j];
-            if (ref_pic->flags & VA_PICTURE_H264_INVALID)
-                continue;
-
-            if (i965_h264_context->fsid_list[i].surface_id == ref_pic->picture_id) {
-                found = 1;
-                break;
-            }
-        }
-
-        if (!found) {
-            struct object_surface *obj_surface = SURFACE(i965_h264_context->fsid_list[i].surface_id);
-            obj_surface->flags &= ~SURFACE_REFERENCED;
-
-            if ((obj_surface->flags & SURFACE_ALL_MASK) == SURFACE_DISPLAYED) {
-                dri_bo_unreference(obj_surface->bo);
-                obj_surface->bo = NULL;
-                obj_surface->flags &= ~SURFACE_REF_DIS_MASK;
-            }
-
-            if (obj_surface->free_private_data)
-                obj_surface->free_private_data(&obj_surface->private_data);
-
-            i965_h264_context->fsid_list[i].surface_id = VA_INVALID_ID;
-            i965_h264_context->fsid_list[i].frame_store_id = -1;
-        }
-    }
-
-    for (i = 0; i < ARRAY_ELEMS(pic_param->ReferenceFrames); i++) {
-        VAPictureH264 *ref_pic = &pic_param->ReferenceFrames[i];
-        int found = 0;
-
-        if (ref_pic->flags & VA_PICTURE_H264_INVALID)
-            continue;
-
-        for (j = 0; j < ARRAY_ELEMS(i965_h264_context->fsid_list); j++) {
-            if (i965_h264_context->fsid_list[j].surface_id == VA_INVALID_ID)
-                continue;
-            
-            if (i965_h264_context->fsid_list[j].surface_id == ref_pic->picture_id) {
-                found = 1;
-                break;
-            }
-        }
-
-        if (!found) {
-            int frame_idx;
-            struct object_surface *obj_surface = SURFACE(ref_pic->picture_id);
-            assert(obj_surface);
-            i965_check_alloc_surface_bo(ctx, obj_surface, 0, VA_FOURCC('N','V','1','2'), SUBSAMPLE_YUV420);
-            
-            for (frame_idx = 0; frame_idx < ARRAY_ELEMS(i965_h264_context->fsid_list); frame_idx++) {
-                for (j = 0; j < ARRAY_ELEMS(i965_h264_context->fsid_list); j++) {
-                    if (i965_h264_context->fsid_list[j].surface_id == VA_INVALID_ID)
-                        continue;
-
-                    if (i965_h264_context->fsid_list[j].frame_store_id == frame_idx)
-                        break;
-                }
-
-                if (j == ARRAY_ELEMS(i965_h264_context->fsid_list))
-                    break;
-            }
-
-            assert(frame_idx < ARRAY_ELEMS(i965_h264_context->fsid_list));
-
-            for (j = 0; j < ARRAY_ELEMS(i965_h264_context->fsid_list); j++) {
-                if (i965_h264_context->fsid_list[j].surface_id == VA_INVALID_ID) {
-                    i965_h264_context->fsid_list[j].surface_id = ref_pic->picture_id;
-                    i965_h264_context->fsid_list[j].frame_store_id = frame_idx;
-                    break;
-                }
-            }
-        }
-    }
-
-    for (i = 0; i < ARRAY_ELEMS(i965_h264_context->fsid_list) - 1; i++) {
-        if (i965_h264_context->fsid_list[i].surface_id != VA_INVALID_ID &&
-            i965_h264_context->fsid_list[i].frame_store_id == i)
-            continue;
-
-        for (j = i + 1; j < ARRAY_ELEMS(i965_h264_context->fsid_list); j++) {
-            if (i965_h264_context->fsid_list[j].surface_id != VA_INVALID_ID &&
-                i965_h264_context->fsid_list[j].frame_store_id == i) {
-                VASurfaceID id = i965_h264_context->fsid_list[i].surface_id;
-                int frame_idx = i965_h264_context->fsid_list[i].frame_store_id;
-
-                i965_h264_context->fsid_list[i].surface_id = i965_h264_context->fsid_list[j].surface_id;
-                i965_h264_context->fsid_list[i].frame_store_id = i965_h264_context->fsid_list[j].frame_store_id;
-                i965_h264_context->fsid_list[j].surface_id = id;
-                i965_h264_context->fsid_list[j].frame_store_id = frame_idx;
-                break;
-            }
-        }
-    }
-}
-
 void 
 i965_avc_bsd_pipeline(VADriverContextP ctx, struct decode_state *decode_state, void *h264_context)
 {
@@ -950,7 +819,7 @@ i965_avc_bsd_pipeline(VADriverContextP ctx, struct decode_state *decode_state, v
 
     assert(decode_state->pic_param && decode_state->pic_param->buffer);
     pic_param = (VAPictureParameterBufferH264 *)decode_state->pic_param->buffer;
-    i965_avc_bsd_frame_store_index(ctx, pic_param, i965_h264_context);
+    intel_update_avc_frame_store_index(ctx, decode_state, pic_param, i965_h264_context->fsid_list);
 
     i965_h264_context->enable_avc_ildb = 0;
     i965_h264_context->picture.i_flag = 1;
@@ -1001,7 +870,7 @@ i965_avc_bsd_pipeline(VADriverContextP ctx, struct decode_state *decode_state, v
                 i965_h264_context->picture.i_flag = 0;
 
             i965_avc_bsd_slice_state(ctx, pic_param, slice_param, i965_h264_context);
-            i965_avc_bsd_buf_base_state(ctx, pic_param, slice_param, i965_h264_context);
+            i965_avc_bsd_buf_base_state(ctx, decode_state, pic_param, slice_param, i965_h264_context);
             i965_avc_bsd_object(ctx, decode_state, pic_param, slice_param, j, i965_h264_context);
             slice_param++;
         }
