@@ -268,6 +268,7 @@ gen5_fill_avc_ref_idx_state(
 )
 {
     unsigned int i, n, frame_idx;
+    int found;
 
     for (i = 0, n = 0; i < ref_list_count; i++) {
         const VAPictureH264 * const va_pic = &ref_list[i];
@@ -275,16 +276,21 @@ gen5_fill_avc_ref_idx_state(
         if (va_pic->flags & VA_PICTURE_H264_INVALID)
             continue;
 
+        found = 0;
         for (frame_idx = 0; frame_idx < MAX_GEN_REFERENCE_FRAMES; frame_idx++) {
             const GenFrameStore * const fs = &frame_store[frame_idx];
             if (fs->surface_id != VA_INVALID_ID &&
                 fs->surface_id == va_pic->picture_id) {
-                assert(frame_idx == fs->frame_store_id);
+                found = 1;
                 break;
             }
         }
-        assert(frame_idx < MAX_GEN_REFERENCE_FRAMES);
-        state[n++] = get_ref_idx_state_1(va_pic, frame_idx);
+
+        if (found) {
+            state[n++] = get_ref_idx_state_1(va_pic, frame_idx);
+        } else {
+            WARN_ONCE("Invalid Slice reference frame list !!!. It is not included in DPB \n");
+        }
     }
 
     for (; n < 32; n++)
@@ -414,6 +420,7 @@ intel_update_avc_frame_store_index(VADriverContextP ctx,
         /* add the new reference frame into the internal DPB */
         if (!found) {
             int frame_idx;
+            int slot_found;
             struct object_surface *obj_surface = decode_state->reference_objects[i];
 
             /* 
@@ -422,60 +429,29 @@ intel_update_avc_frame_store_index(VADriverContextP ctx,
              */
             i965_check_alloc_surface_bo(ctx, obj_surface, 1, VA_FOURCC('N', 'V', '1', '2'), SUBSAMPLE_YUV420);
 
+            slot_found = 0;
+            frame_idx = -1;
             /* Find a free frame store index */
-            for (frame_idx = 0; frame_idx < MAX_GEN_REFERENCE_FRAMES; frame_idx++) {
-                for (j = 0; j < MAX_GEN_REFERENCE_FRAMES; j++) {
-                    if (frame_store[j].surface_id == VA_INVALID_ID ||
-                        frame_store[j].obj_surface == NULL)
-                        continue;
-
-                    if (frame_store[j].frame_store_id == frame_idx) /* the store index is in use */
-                        break;
-                }
-
-                if (j == MAX_GEN_REFERENCE_FRAMES)
-                    break;
-            }
-
-            assert(frame_idx < MAX_GEN_REFERENCE_FRAMES);
-
             for (j = 0; j < MAX_GEN_REFERENCE_FRAMES; j++) {
                 if (frame_store[j].surface_id == VA_INVALID_ID ||
                     frame_store[j].obj_surface == NULL) {
-                    frame_store[j].surface_id = ref_pic->picture_id;
-                    frame_store[j].frame_store_id = frame_idx;
-                    frame_store[j].obj_surface = obj_surface;
+                    frame_idx = j;
+                    slot_found = 1;
                     break;
                 }
             }
-        }
-    }
 
-    /* sort */
-    for (i = 0; i < MAX_GEN_REFERENCE_FRAMES - 1; i++) {
-        if (frame_store[i].surface_id != VA_INVALID_ID &&
-            frame_store[i].obj_surface != NULL &&
-            frame_store[i].frame_store_id == i)
-            continue;
 
-        for (j = i + 1; j < MAX_GEN_REFERENCE_FRAMES; j++) {
-            if (frame_store[j].surface_id != VA_INVALID_ID &&
-                frame_store[j].obj_surface != NULL &&
-                frame_store[j].frame_store_id == i) {
-                VASurfaceID id = frame_store[i].surface_id;
-                int frame_idx = frame_store[i].frame_store_id;
-                struct object_surface *obj_surface = frame_store[i].obj_surface;
-
-                frame_store[i].surface_id = frame_store[j].surface_id;
-                frame_store[i].frame_store_id = frame_store[j].frame_store_id;
-                frame_store[i].obj_surface = frame_store[j].obj_surface;
-                frame_store[j].surface_id = id;
+	    if (slot_found) {
+                frame_store[j].surface_id = ref_pic->picture_id;
                 frame_store[j].frame_store_id = frame_idx;
                 frame_store[j].obj_surface = obj_surface;
-                break;
-            }
+            } else {
+                WARN_ONCE("Not free slot for DPB reference list!!!\n");
+	    }
         }
     }
+
 }
 
 void
@@ -517,8 +493,61 @@ intel_update_vc1_frame_store_index(VADriverContextP ctx,
 
 }
 
+void
+intel_update_vp8_frame_store_index(VADriverContextP ctx,
+                                   struct decode_state *decode_state,
+                                   VAPictureParameterBufferVP8 *pic_param,
+                                   GenFrameStore frame_store[MAX_GEN_REFERENCE_FRAMES])
+{
+    struct object_surface *obj_surface;
+    int i;
+
+    obj_surface = decode_state->reference_objects[0];
+
+    if (pic_param->last_ref_frame == VA_INVALID_ID ||
+        !obj_surface ||
+        !obj_surface->bo) {
+        frame_store[0].surface_id = VA_INVALID_ID;
+        frame_store[0].obj_surface = NULL;
+    } else {
+        frame_store[0].surface_id = pic_param->last_ref_frame;
+        frame_store[0].obj_surface = obj_surface;
+    }
+
+    obj_surface = decode_state->reference_objects[1];
+
+    if (pic_param->golden_ref_frame == VA_INVALID_ID ||
+        !obj_surface ||
+        !obj_surface->bo) {
+        frame_store[1].surface_id = frame_store[0].surface_id;
+        frame_store[1].obj_surface = frame_store[0].obj_surface;
+    } else {
+        frame_store[1].surface_id = pic_param->golden_ref_frame;
+        frame_store[1].obj_surface = obj_surface;
+    }
+
+    obj_surface = decode_state->reference_objects[2];
+
+    if (pic_param->alt_ref_frame == VA_INVALID_ID ||
+        !obj_surface ||
+        !obj_surface->bo) {
+        frame_store[2].surface_id = frame_store[0].surface_id;
+        frame_store[2].obj_surface = frame_store[0].obj_surface;
+    } else {
+        frame_store[2].surface_id = pic_param->alt_ref_frame;
+        frame_store[2].obj_surface = obj_surface;
+    }
+
+    for (i = 3; i < MAX_GEN_REFERENCE_FRAMES; i++) {
+        frame_store[i].surface_id = frame_store[i % 2].surface_id;
+        frame_store[i].obj_surface = frame_store[i % 2].obj_surface;
+    }
+
+}
+
 static VAStatus
 intel_decoder_check_avc_parameter(VADriverContextP ctx,
+                                  VAProfile h264_profile,
                                   struct decode_state *decode_state)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
@@ -538,6 +567,14 @@ intel_decoder_check_avc_parameter(VADriverContextP ctx,
     if (pic_param->CurrPic.picture_id != decode_state->current_render_target)
         goto error;
 
+    if ((h264_profile != VAProfileH264Baseline)) {
+       if (pic_param->num_slice_groups_minus1 ||
+           pic_param->pic_fields.bits.redundant_pic_cnt_present_flag) {
+           WARN_ONCE("Unsupported the FMO/ASO constraints!!!\n");
+           goto error;
+       }
+    }
+
     for (i = 0; i < 16; i++) {
         if (pic_param->ReferenceFrames[i].flags & VA_PICTURE_H264_INVALID ||
             pic_param->ReferenceFrames[i].picture_id == VA_INVALID_SURFACE)
@@ -654,6 +691,48 @@ error:
     return VA_STATUS_ERROR_INVALID_PARAMETER;
 }
 
+static VAStatus
+intel_decoder_check_vp8_parameter(VADriverContextP ctx,
+                                  struct decode_state *decode_state)
+{
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    VAPictureParameterBufferVP8 *pic_param = (VAPictureParameterBufferVP8 *)decode_state->pic_param->buffer;
+    struct object_surface *obj_surface;	
+    int i = 0;
+
+    if (pic_param->last_ref_frame != VA_INVALID_SURFACE) {
+        obj_surface = SURFACE(pic_param->last_ref_frame);
+
+        if (obj_surface && obj_surface->bo)
+            decode_state->reference_objects[i++] = obj_surface;
+        else
+            decode_state->reference_objects[i++] = NULL;
+    }
+
+    if (pic_param->golden_ref_frame != VA_INVALID_SURFACE) {
+        obj_surface = SURFACE(pic_param->golden_ref_frame);
+
+        if (obj_surface && obj_surface->bo)
+            decode_state->reference_objects[i++] = obj_surface;
+        else
+            decode_state->reference_objects[i++] = NULL;
+    }
+
+    if (pic_param->alt_ref_frame != VA_INVALID_SURFACE) {
+        obj_surface = SURFACE(pic_param->alt_ref_frame);
+
+        if (obj_surface && obj_surface->bo)
+            decode_state->reference_objects[i++] = obj_surface;
+        else
+            decode_state->reference_objects[i++] = NULL;
+    }
+
+    for ( ; i < 16; i++)
+        decode_state->reference_objects[i] = NULL;
+
+    return VA_STATUS_SUCCESS;
+}
+
 VAStatus
 intel_decoder_sanity_check_input(VADriverContextP ctx,
                                  VAProfile profile,
@@ -679,10 +758,10 @@ intel_decoder_sanity_check_input(VADriverContextP ctx,
         vaStatus = intel_decoder_check_mpeg2_parameter(ctx, decode_state);
         break;
         
-    case VAProfileH264Baseline:
+    case VAProfileH264ConstrainedBaseline:
     case VAProfileH264Main:
     case VAProfileH264High:
-        vaStatus = intel_decoder_check_avc_parameter(ctx, decode_state);
+        vaStatus = intel_decoder_check_avc_parameter(ctx, profile, decode_state);
         break;
 
     case VAProfileVC1Simple:
@@ -695,6 +774,10 @@ intel_decoder_sanity_check_input(VADriverContextP ctx,
         vaStatus = VA_STATUS_SUCCESS;
         break;
 
+    case VAProfileVP8Version0_3:
+        vaStatus = intel_decoder_check_vp8_parameter(ctx, decode_state);
+        break;
+
     default:
         vaStatus = VA_STATUS_ERROR_INVALID_PARAMETER;
         break;
@@ -703,3 +786,44 @@ intel_decoder_sanity_check_input(VADriverContextP ctx,
 out:
     return vaStatus;
 }
+
+/*
+ * Return the next slice paramter
+ *
+ * Input:
+ *      slice_param: the current slice
+ *      *group_idx & *element_idx the current slice position in slice groups
+ * Output:
+ *      Return the next slice parameter
+ *      *group_idx & *element_idx the next slice position in slice groups,
+ *      if the next slice is NULL, *group_idx & *element_idx will be ignored
+ */
+VASliceParameterBufferMPEG2 *
+intel_mpeg2_find_next_slice(struct decode_state *decode_state,
+                            VAPictureParameterBufferMPEG2 *pic_param,
+                            VASliceParameterBufferMPEG2 *slice_param,
+                            int *group_idx,
+                            int *element_idx)
+{
+    VASliceParameterBufferMPEG2 *next_slice_param;
+    unsigned int width_in_mbs = ALIGN(pic_param->horizontal_size, 16) / 16;
+    int j = *group_idx, i = *element_idx + 1;
+
+    for (; j < decode_state->num_slice_params; j++) {
+        for (; i < decode_state->slice_params[j]->num_elements; i++) {
+            next_slice_param = ((VASliceParameterBufferMPEG2 *)decode_state->slice_params[j]->buffer) + i;
+
+            if ((next_slice_param->slice_vertical_position * width_in_mbs + next_slice_param->slice_horizontal_position) >=
+                (slice_param->slice_vertical_position * width_in_mbs + slice_param->slice_horizontal_position)) {
+                *group_idx = j;
+                *element_idx = i;
+
+                return next_slice_param;
+            }
+        }
+
+        i = 0;
+    }
+
+    return NULL;
+}
