@@ -29,26 +29,41 @@
 #ifndef _I965_ENCODER_H_
 #define _I965_ENCODER_H_
 
-#include <xf86drm.h>
 #include <drm.h>
 #include <i915_drm.h>
 #include <intel_bufmgr.h>
+
 #include "i965_structs.h"
 #include "i965_drv_video.h"
 
-#include "gen6_vme.h"
-#include "gen6_mfc.h"
-
-struct gen6_encoder_context
+struct intel_encoder_context
 {
     struct hw_context base;
-    struct gen6_vme_context vme_context;
-    struct gen6_mfc_context mfc_context;
+    int codec;
+    VASurfaceID input_yuv_surface;
+    int is_tmp_id;
+    unsigned int rate_control_mode;
+    void *vme_context;
+    void *mfc_context;
+    void (*vme_context_destroy)(void *vme_context);
+    VAStatus (*vme_pipeline)(VADriverContextP ctx,
+                             VAProfile profile,
+                             struct encode_state *encode_state,
+                             struct intel_encoder_context *encoder_context);
+    void (*mfc_context_destroy)(void *mfc_context);
+    VAStatus (*mfc_pipeline)(VADriverContextP ctx,
+                             VAProfile profile,
+                             struct encode_state *encode_state,
+                             struct intel_encoder_context *encoder_context);
+    void (*mfc_brc_prepare)(struct encode_state *encode_state,
+                            struct intel_encoder_context *encoder_context);
 };
 
 extern struct hw_context *
-gen75_enc_hw_context_init(VADriverContextP ctx, VAProfile profile);
+gen75_enc_hw_context_init(VADriverContextP ctx, struct object_config *obj_config);
 
+extern struct hw_context *
+gen8_enc_hw_context_init(VADriverContextP ctx, struct object_config *obj_config);
 #endif	/* _I965_ENCODER_H_ */
 
 
