@@ -28,6 +28,9 @@
  */
 
 #include "sysdeps.h"
+
+#include <va/va_drmcommon.h>
+
 #include "intel_batchbuffer.h"
 #include "intel_memman.h"
 #include "intel_driver.h"
@@ -64,12 +67,12 @@ static void intel_driver_get_revid(struct intel_driver_data *intel, int *value)
 	return;
 }
 
-Bool 
+bool 
 intel_driver_init(VADriverContextP ctx)
 {
     struct intel_driver_data *intel = intel_driver_data(ctx);
     struct drm_state * const drm_state = (struct drm_state *)ctx->drm_state;
-    int has_exec2, has_bsd, has_blt;
+    int has_exec2 = 0, has_bsd = 0, has_blt = 0, has_vebox = 0;
 
     assert(drm_state);
     assert(VA_CHECK_DRM_AUTH_TYPE(ctx, VA_DRM_AUTH_DRI1) ||
@@ -81,7 +84,7 @@ intel_driver_init(VADriverContextP ctx)
                           VA_CHECK_DRM_AUTH_TYPE(ctx, VA_DRM_AUTH_CUSTOM));
 
     if (!intel->dri2Enabled) {
-        return False;
+        return false;
     }
 
     intel->locked = 0;
@@ -94,19 +97,19 @@ intel_driver_init(VADriverContextP ctx)
         intel->has_bsd = has_bsd;
     if (intel_driver_get_param(intel, I915_PARAM_HAS_BLT, &has_blt))
         intel->has_blt = has_blt;
+    if (intel_driver_get_param(intel, I915_PARAM_HAS_VEBOX, &has_vebox))
+        intel->has_vebox = !!has_vebox;
    
     intel_driver_get_revid(intel, &intel->revision);
     intel_memman_init(intel);
-    return True;
+    return true;
 }
 
-Bool 
+void 
 intel_driver_terminate(VADriverContextP ctx)
 {
     struct intel_driver_data *intel = intel_driver_data(ctx);
 
     intel_memman_terminate(intel);
     pthread_mutex_destroy(&intel->ctxmutex);
-
-    return True;
 }
